/*
 *      Copyright (c) 2018-2028, Chill Zhuang All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions are met:
 *
 *  Redistributions of source code must retain the above copyright notice,
 *  this list of conditions and the following disclaimer.
 *  Redistributions in binary form must reproduce the above copyright
 *  notice, this list of conditions and the following disclaimer in the
 *  documentation and/or other materials provided with the distribution.
 *  Neither the name of the dreamlu.net developer nor the names of its
 *  contributors may be used to endorse or promote products derived from
 *  this software without specific prior written permission.
 *  Author: Chill 庄骞 (smallchill@163.com)
 */
package org.springblade.modules.finance.vo;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springblade.common.enums.AfterSalesStatusEnum;
import org.springblade.common.enums.AfterSalesTypeEnum;
import org.springblade.modules.finance.entity.AfterSales;
import org.springblade.modules.finance.entity.AfterSalesGoods;
import org.springblade.modules.finance.entity.OrderGoods;
import org.springblade.modules.finance.entity.OrderRefund;
import org.springblade.modules.mall.vo.StoreVO;

import java.util.List;

/**
 * 售后表视图实体类
 *
 * @author BladeX
 * @since 2023-06-15
 */
@Data
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "AfterSalesVO对象", description = "售后表")
public class AfterSalesVO extends AfterSales {
	private static final long serialVersionUID = 1L;

	@ApiModelProperty(value = "售后类型")
	private String typeDesc;

	@ApiModelProperty(value = "售后状态")
	private String statusDesc;

	@ApiModelProperty(value = "订单商品信息")
	private List<OrderGoods> goodsList;

	@ApiModelProperty(value = "退款商品信息")
	private List<OrderGoodsVO> refundGoodsList;

	@ApiModelProperty(value = "退款商品信息")
	private List<AfterSalesGoods> afterSalesGoodsList;

	@ApiModelProperty(value = "退款信息")
	private OrderRefund refund;

	@ApiModelProperty(value = "订单信息")
	private OrderVO order;

	@ApiModelProperty(value = "商家信息")
	private StoreVO store;

	@ApiModelProperty(value = "订单编号")
	private String orderNo;

	@ApiModelProperty(value = "自提点id",hidden = true)
	private Long selfTakeId;

	@ApiModelProperty(value = "同意退款和拒绝退款按钮显示，0不显示，1显示")
	private Integer disposeBtn = 0;

	public String getStatusDesc() {
		return AfterSalesStatusEnum.getNameByValue(getStatus());
	}

	public String getTypeDesc() {
		return AfterSalesTypeEnum.getNameByValue(getType());
	}

	public Integer getDisposeBtn() {
       if (super.getStatus() == 0 && super.getRefundState() != 2){
           return 1;
       } else {
           return 0;
       }
	}

}
