package org.springblade.modules.quartz.task;

import cn.hutool.core.collection.CollUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.time.DateUtils;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springblade.common.constant.CacheBizConstant;
import org.springblade.common.enums.OrderStateEnum;
import org.springblade.core.redis.cache.BladeRedis;
import org.springblade.modules.finance.entity.Order;
import org.springblade.modules.finance.service.IOrderService;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * Author pangqijun
 * Date 2023/7/19
 * Description
 */
@Slf4j
@Component
public class OrderTimeoutTask {

    @Lazy
    @Resource
    private IOrderService orderService;
    @Resource
    private BladeRedis bladeRedis;
    @Resource
    private RedissonClient redissonClient;


    public void task() {
        RLock lock = redissonClient.getLock("order:timeout:local");
        lock.lock(10, TimeUnit.SECONDS);
        try {
            List<Order> list = orderService.lambdaQuery()
                    .eq(Order::getOrderState, OrderStateEnum.WAIT_PAY.getValue())
                    .lt(Order::getCreateTime, DateUtils.addMinutes(new Date(), -30))
                    .list();
            if (CollUtil.isNotEmpty(list)) {
                for (Order next : list) {
                    log.info("处理超时订单,orderId[{}]", next.getId());
                    orderService.orderTimeout(next.getId(), null);
                    bladeRedis.zRem(CacheBizConstant.TIMEOUT_ORDER_LIST, next.getId());
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error(e.getMessage(), e);
        } finally {
            if (lock.isLocked()) {
                lock.unlock();
            }
        }
    }
}
